;NESA File Player by Chris Covell.
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

   ORG   $8000    ;32Kb PRG-ROM, 8Kb CHR-ROM

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM.
   ldy #$06    ;To clear 7 x $100 bytes, from $000 to $6FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0


;****** SET UP SCREEN **************************************

   lda   #$3F
   sta   $2006
   lda   #$00
   sta   $2006
   ldy   #$20
   ldx   #$0
.SetPalette   lda   .CMAP,X
   sta   $2007
   inx
   dey
   bne   .SetPalette

   lda   #$23
   sta   $2006
   lda   #$C0
   sta   $2006
   ldy   #$40
   ldx   #$0
.SetAttr lda   .AttrMap,X
   sta   $2007
   inx
   dey
   bne   .SetAttr

   lda   #$20
   sta   $2006
   sta   $2006
   ldx   #$0
   ldy   #$40
.SetName1 lda .NameTable1,X
   sta   $2007
   inx
   dey
   bne   .SetName1

   lda   #$20
   sta   $2006
   lda   #$C0
   sta   $2006
   ldx   #$0
   ldy   #$60
.SetMeter1 lda .MeterTable,X
   sta   $2007
   inx
   dey
   bne   .SetMeter1

;****** SET UP AUDIO *****************************************

;----- Read header first & set up memory -----------------

   lda   #$00                          ;CHANGE THIS!!!!!!
   sta   $10
   lda   #$84                          ;CHANGE THIS!!!!!!
   sta   $11

   jsr   IncAddr                       ;Point to "E"
   jsr   IncAddr                       ;Point to "S"
   jsr   IncAddr                       ;Point to "A"
   jsr   IncAddr                       ;Point to NESAVersion
   ldx   #$0
   lda   ($10,X)
   and   #$0F                          ;To get Version
   cmp   #$02
   beq   .VersionsGood
   jmp   .Loop
.VersionsGood lda ($10,X)
   and   #$F0                          ;To get Compression Type
   cmp   #$20
   beq   .TypesGood
   jmp   .Loop
.TypesGood
   jsr   IncAddr                       ;Point to NESASndControl
   lda   ($10,X)
   sta   $4015                         ;Save channels to Channel Control

   jsr   IncAddr                       ;Point to NESAControl1
   lda   ($10,X)                       ;Load Control into A
   sta   $30                           ;Save Control into $30

   jsr   IncAddr                       ;Point to NESAControl2
   jsr   IncAddr                       ;Point to NESAControl3
   jsr   IncAddr                       ;Point to NESAControl4
   jsr   IncAddr                       ;Point to NESAControl5
   jsr   IncAddr                       ;Point to NESAControl6
   jsr   IncAddr                       ;Point to NESALength
   lda   ($10,X)
   sta   $20                           ;Save length in $20 - $23
   jsr   IncAddr
   lda   ($10,X)
   sta   $21
   jsr   IncAddr
   lda   ($10,X)
   sta   $22
   jsr   IncAddr
   lda   ($10,X)
   sta   $23
   jsr   IncAddr                       ;Point to beginning of Data!

   ldx   #$0
   lda   $30                           ;Load up SNDControl
   and   #%01000000                    ;Is there Title text?
   beq   .NoTitle
   jsr   DisplayTitle
.NoTitle lda $30
   and   #%00100000                    ;Is there Comment text?
   beq   .NoComment                    ;I like this label. :-)
   jsr   DisplayComment
.NoComment

   lda   #%10000000
   sta   $2000
   lda   #%00001110
   sta   $2001

.MainLoop
   ldx   #$0
   lda   ($10,X)                       ;Load Byte 1 of data
   lsr
   lsr
   lsr
   lsr                                 ;Isolate register to write
   sta   $31                           ;Store register in $31
   lda   ($10,X)
   and   #$0F                          ;Isolate upper ticks.
   sta   $41                           ;Store upper ticks in $41

   jsr   IncAddr
   lda   ($10,X)                       ;Load lower ticks
   sta   $40                           ;Store lower ticks in $40

   jsr   IncAddr
   lda   ($10,X)                       ;Load register data
   sta   $52
.WaitForVBs ldy #$00
   cpy   $40                           ;Is $40 zero?
   bne   .WaitForVBs
   cpy   $41                           ;Is $41 zero?
   bne   .WaitForVBs

   ldx   $31                           ;load register number into X
   sta   $4000,X                       ;Save to appropriate sound register.
   ldx   #$00
   cpx   $31
   bne   .NoVolume1
   sta   $32
   jmp   .NoVolume
.NoVolume1   ldx   #$04
   cpx   $31
   bne   .NoVolume2
   sta   $33
   jmp   .NoVolume
.NoVolume2   ldx   #$08
   cpx   $31
   bne   .NoVolume
   sta   $34
.NoVolume   jsr   IncAddr                       ;Point to new triplet of data.
   jsr   DecLength
   jsr   DecLength
   jsr   DecLength

   lda   #$0
   cmp   $23
   bne   .MainLoop
   cmp   $22
   bne   .MainLoop
   cmp   $21
   bne   .MainLoop
   cmp   $20
   bne   .MainLoop

;--- END OF AUDIO ----------------------

;Now just loop forever?
   lda   #$0
;   sta   $2001
   sta   $4015
.Loop jmp    .Loop

.CMAP dc.b  #$0D,#$09,#$19,#$39,#$0D,#$08,#$18,#$38,#$0D,#$07,#$17,#$37,#$0D,#$06,#$16,#$36
      dc.b  #$0D,#$30,#$30,#$30,#$0D,#$30,#$30,#$30,#$0D,#$30,#$30,#$30,#$0D,#$30,#$30,#$30

.AttrMap dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00
         dc.b #$00,#$00,#$00,#$50,#$A0,#$F0,#$00,#$00
         dc.b #$00,#$00,#$00,#$05,#$0A,#$0F,#$00,#$00
         dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00
         dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00
         dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00
         dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00
         dc.b #$00,#$00,#$00,#$00,#$00,#$00,#$00,#$00

;                 "                                "
.NameTable1 dc.b  "    NESA Audio Format Player    "
            dc.b  "\1998 Chris C. ccovell@direct.ca"

.MeterTable dc.b  " CHN0: -                +       "
            dc.b  " CHN1: -                +       "
            dc.b  " CHN2: -                +       "

DisplayTitle SUBROUTINE
   ldy   #$20
   lda   #$20
   sta   $2006
   lda   #$80
   sta   $2006
.TitleLoop lda ($10,X)
   sta   $2007
   jsr   IncAddr
   dey
   bne   .TitleLoop
   rts

DisplayComment SUBROUTINE
   ldy   #$20
   ldy   #$20
   lda   #$20
   sta   $2006
   lda   #$A0
   sta   $2006
.CommentLoop lda ($10,X)
   sta   $2007
   jsr   IncAddr
   dey
   bne   .CommentLoop
   rts
   rts

IncAddr SUBROUTINE
   inc   $10
   bne   .NoOverflow
   inc   $11
.NoOverflow rts


DecLength SUBROUTINE
   pha
   tya
   pha

   lda   $23
   sec               ;Make sure *no borrow* initially!
   sbc   #$01        ;Decrement memory.
   sta   $23
   lda   $22
   sbc   #$00
   sta   $22
   lda   $21
   sbc   #$00
   sta   $21
   lda   $20
   sbc   #$00
   sta   $21

   pla
   tay
   pla
   rts

DisplayVolume SUBROUTINE

   pha
   tya
   pha

   lda   #$20
   sta   $2006
   lda   #$C8
   sta   $2006
   ldy   #$10                 ;Clear a line of 16 blocks
   lda   #$00
.ClearVol1 sta $2007
   dey
   bne   .ClearVol1

   lda   #$20
   sta   $2006
   lda   #$E8
   sta   $2006
   ldy   #$10                 ;Clear a line of 16 blocks
   lda   #$00
.ClearVol2 sta $2007
   dey
   bne   .ClearVol2

   lda   #$21
   sta   $2006
   lda   #$08
   sta   $2006
   ldy   #$10                 ;Clear a line of 16 blocks
   lda   #$00
.ClearVol3 sta $2007
   dey
   bne   .ClearVol3

   lda   $32
   and   #$0F
   tax
   inx
   lda   #$20
   sta   $2006
   lda   #$C8
   sta   $2006
   lda   #$03
.MakeBar1 sta   $2007
   dex
   bne   .MakeBar1

   lda   $33
   and   #$0F
   tax
   inx
   lda   #$20
   sta   $2006
   lda   #$E8
   sta   $2006
   lda   #$03
.MakeBar2 sta   $2007
   dex
   bne   .MakeBar2

   lda   $34
   and   #$0F
   tax
   inx
   lda   #$21
   sta   $2006
   lda   #$08
   sta   $2006
   lda   #$03
.MakeBar3 sta   $2007
   dex
   bne   .MakeBar3

   pla
   tay
   pla

   rts

NMI_Routine SUBROUTINE

   pha
   tya
   pha

   jsr   DisplayVolume
   ldy   #$00
   cpy   $40      ;Is the "tick counter" 0?
   bne   .DecTicks
   cpy   $41      ;Is the "tick counter" 0?
   beq   .NoTicks
.DecTicks
   ldy   #$00
   cpy   $40      ;Is the "tick counter" 0?
   bne   .NormalDecrease
   dec   $41
.NormalDecrease   dec   $40

.NoTicks

   lda   #$00
   sta   $2005
   sta   $2005
   pla
   tay
   pla
   rti

IRQ_Routine       ;Dummy label
   rti

   ORG   $8400,0                       ;CHANGE THIS!!!!!!!
.NESAHeader       dc.b  "NESA"
.NESAVersion      dc.b  #%00100010
.NESASndControl   dc.b  #%00000111
.NESAControl1     dc.b  #%01100000
.NESAControl2     dc.b  #%00000000
.NESAControl3     dc.b  #%00000000
.NESAControl4     dc.b  #%00000000
.NESAControl5     dc.b  #%00000000
.NESAControl6     dc.b  #%00000000
.NESALength       dc.b  #0,#0,#0,#$45
.NESA            ;      "                                "
                  dc.b  "This is a title field!  !  !!!!!"
                  dc.b  "This is a comment field!  !!!!!!"
                  dc.b  #$00,#$00,#%10111111,#$10,#$00,#$00,#$20,#$00,#$80,#$30,#$00,#$FF
                  dc.b  #$00,#$1E,#%10111111,#$00,#$1E,#%10111110,#$00,#$1E,#%10111101
                  dc.b  #$00,#$1E,#%10111100,#$00,#$1E,#%10111011,#$00,#$1E,#%10111010
                  dc.b  #$00,#$1E,#%10111001,#$00,#$1E,#%10111000
                  dc.b  #$00,#$1E,#%10110111,#$00,#$1E,#%10110110,#$00,#$1E,#%10110101
                  dc.b  #$00,#$1E,#%10110100,#$00,#$1E,#%10110011,#$00,#$1E,#%10110010
                  dc.b  #$00,#$1E,#%10110001,#$00,#$1E,#%10110000
                  dc.b  #$40,#$00,#$FF,#$80,#$00,#$FF,#$00,#$00,#%10111111

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
